<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: AllocateZoneImpressions.php 3416 2005-06-30 14:40:49Z andrew $
*/

require_once MAX_PATH . '/lib/max/Dal/Forecasting.php';
require_once MAX_PATH . '/lib/max/Maintenance/Forecasting/Channel/Limitations.php';

/**
 * A class to create a limitation for channel forecasting
 *
 * Note: Always do the multiplication before the division in the code, to try to 
 * avoid rounding errors.
 * 
 * @package    MaxMaintenance
 * @subpackage Forecasting
 * @author     Radek Maciaszek <radek@m3.net>
 */
class MAX_Maintenance_Forecasting_Channel_ChannelLimitations
{
    var $oLimitations;
    var $aLimitations;
    
    var $aSqlLimitations;
    var $phpLimitations;
    
    /**
     * The constructor method.
     */
    function MAX_Maintenance_Forecasting_Channel_ChannelLimitations()
    {
        $this->_init();
    }
    
    /**
     * A private method to initialise the class. Set the configuration file
     * data, the current date, and details of when the Maintenance Statistics
     * and Maintenance Prioritisation engines last ran.
     * 
     * @access private
     */
    function _init()
    {
        $this->oLimitations = new MAX_Maintenance_Forecasting_Channel_Limitations();
        $this->oDal = & $this->_getDal();
    }
    
    /**
     * Method to create/register/return the Maintenance Data Access Layer.
     *
     * @access private
     */
    function &_getDal()
    {
        $oServiceLocator = &ServiceLocator::instance();
        $oDal = $oServiceLocator->get('MAX_Dal_Forecasting');
        if (!$oDal) {
            $oDal = &MAX_Dal_Forecasting::singleton();
            $oServiceLocator->register('MAX_Dal_Forecasting', $oDal);
        }
        return $oDal;
    }
    
    /**
     * Return sql limitations
     * @param array $zonesIds  Zones IDs - if zone id is specified sql limitation for query
     *                     is also returned
     * @return array  Array containing SQL limitations in format:
     *                [0] => array (
     *                             [0] => 'any sql query limitation..',
     *                             [1] => 'comparison: either "or" or "and"'
     *                       ),
     *                [1] => array ....
     */
    function getSqlLimitations($zonesIds = array())
    {
        if(!empty($zonesIds)) {
            $sqlLimitations[] = array('zone_id IN ('.implode(',', $zonesIds).')', 'and');
            foreach($this->aSqlLimitations as $limitation) {
                $sqlLimitations[] = $limitation;
            }
            return $sqlLimitations;
        }
        return $this->aSqlLimitations;
    }
    
    /**
     * Return php limitations
     * @return string  String containing php limitations which will be run
     *                 later by using eval() function
     */
    function getPhpLimitations()
    {
        return $this->phpLimitations;
    }
    
    /**
     * Create SQL limitations
     *
     * @param array $limitations  Array of limitations
     * @param boolean $removeUsedLimitations  If true unset used limitations from $limitations array
     */
    function buildSqlLimitations(&$limitations, $removeUsedLimitations = true)
    {
        $this->aSqlLimitations = array();
        foreach ($limitations as $key => $limitation) {
            if ($this->oLimitations->isSqlLimitationExists($limitation['type'])) {
                $sqlLimitation = $this->oLimitations->getSqlLimitation($limitation['type'], $limitation['data'],
                        $limitation['comparison']);
                // Do not add if always true (eg. when data == '*')
                if($sqlLimitation !== true) {
                    $this->aSqlLimitations[] = array($sqlLimitation, $limitation['logical']);
                }
                if($removeUsedLimitations) {
                    unset($limitations[$key]);
                }
            }
        }
    }

    /**
     * Create PHP limitations
     *
     * @param array $limitations  Array of limitations
     *
     * @return boolean - True on success else false
     */
    function buildPhpLimitations($limitations)
    {
        $this->phpLimitations = '';
        foreach ($limitations as $limitation) {
            if(!empty($this->phpLimitations)) {
                $this->phpLimitations .= ' '.$limitation['logical'].' ';
            }
            $this->phpLimitations .= $this->oLimitations->getPhpLimitation($limitation['type'], 
                $limitation['data'], $limitation['comparison']);
        }
    }

    /**
     * Method get limitations for channel and build SQL and/or 
     * PHP limitations for that channel
     *
     * @param integer $channelId  Channel ID
     *
     * @return boolean - True on success else false
     */
    function buildLimitations($channelId)
    {
        $this->phpLimitations = '';
        $this->aSqlLimitations = array();
        
        $this->aLimitations = $aLimitations = $this->oDal->getChannelLimitations($channelId);
        if ($aLimitations == false) {
            return false;
        }
        $allSqlLimitationsExists = true;
        $orExists = false;
        
        foreach ($aLimitations as $limitation) {
            if(!$this->oLimitations->isSqlLimitationExists($limitation['type'])) {
                $allSqlLimitationsExists = false;
            }
            // Check if any 'OR' exists in any limitation
            if(strtoupper($limitation['logical']) == 'OR') {
                $orExists = true;
            }
        }
        
        // Build SQL limitation if all SQL equivalents exists or if there is no 'OR's in limitations
        if($allSqlLimitationsExists || !$orExists) {
            $this->buildSqlLimitations($aLimitations, true);
        }
        if($allSqlLimitationsExists) {
            // do not build php limitations - just return true
            return true;
        }
        
        $this->buildPhpLimitations($aLimitations);
        
        return true;
    }

}

?>